Package: hyreg2
Type: Package
Title: Estimate Latent Classes on a Mixture of Continuous and
        Dichotomous Data
Version: 1.1.2
Authors@R: c(person(given = "Svenja",
                      family = "Elkenkamp",
                      role = c("aut", "cre"),
                      email = "svenja.elkenkamp@uni-bielefeld.de"),
               person(given = "Kim",
                      family = "Rand",
                      role = "aut"),
               person(given = "John",
                      family = "Grosser",
                      role = "aut"),
               person("EuroQol",
                      role = "fnd"))
Description: The hybrid model likelihood as described by Ramos-Goñi et al. (2017) <doi:10.1097/MLR.0000000000000283> is implemented and
      and embedded in a latent class framework. The package is based on 'flexmix' and among others contains an M-step-driver as described by Leisch (2004) <doi:10.18637/jss.v011.i08>.
      Users can, for example, estimate latent classes for EQ-5D value sets and address preference heterogeneity. Both uncensored and censored data are supported. 
      Furthermore, heteroscedasticity can be taken into account. It is possible to control for different covariates on the continuous and dichotomous  data and start values can differ
      between the expected latent classes. 
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1
Imports: flexmix, bbmle, ggplot2, methods, stats, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 2.10)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-03-17 11:53:06 UTC; selkenkamp
Author: Svenja Elkenkamp [aut, cre],
  Kim Rand [aut],
  John Grosser [aut],
  EuroQol [fnd]
Maintainer: Svenja Elkenkamp <svenja.elkenkamp@uni-bielefeld.de>
Repository: CRAN
Date/Publication: 2026-03-17 12:40:09 UTC
Built: R 4.6.0; ; 2026-04-28 05:05:58 UTC; unix
