# Generated by roxygen2: do not edit by hand

export(FLXMRhyreg)
export(FLXMRhyreg_het)
export(get_stv)
export(give_class)
export(hyreg2)
export(hyreg2_het)
export(plot_hyreg2)
export(summary_hyreg2)
export(the)
importFrom(bbmle,mle2)
importFrom(bbmle,summary)
importFrom(flexmix,flexmix)
importFrom(ggplot2,ggplot)
importFrom(methods,new)
importFrom(stats,as.formula)
importFrom(stats,dnorm)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,setNames)
importFrom(utils,globalVariables)
