% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_data_mo}
\alias{simulated_data_mo}
\title{simulated_data_mo}
\format{
\subsection{\code{simulated_data_mo}}{

A simulated data frame with 480 rows and 9 columns, following a combination of normal and binomial distribution
\describe{
\item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
\item{y}{result of the formula y ~ -1 + mo2 + mo3 + mo4 + mo5}
\item{mo2, mo3, mo4, mo5}{dummy variables}
\item{class}{original class of the data point}
\item{id}{id number of observations to simulated different persons}
\item{y_cens}{column y censored at 0 (lower boundary)}
...
}
}
}
\source{
{simulated with true parameter values:
Class 1: sigma  = 0.001, theta = 0.2 and c(mo2,mo3,mo4,mo5) = c(0.005, 0.01, 0.08, 0.1)
Class 2: sigma  = 0.1, theta = 2 and c(mo2,mo3,mo4,mo5) = c(0.2, 0.4, 0.6, 0.8)}
}
\usage{
simulated_data_mo
}
\description{
simulated_data_mo
}
\keyword{datasets}
